@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2008-2009 Pierre-Luc Beaudoin <pierre-luc@pierlux.com>
+ * Copyright (C) 2011 Jiri Techet <techet@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -43,7 +44,8 @@
 
 G_DEFINE_TYPE (ChamplainPathLayer, champlain_path_layer, CHAMPLAIN_TYPE_LAYER)
 
-#define GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CHAMPLAIN_TYPE_PATH_LAYER, ChamplainPathLayerPrivate))
+#define GET_PRIVATE(obj) \
+  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CHAMPLAIN_TYPE_PATH_LAYER, ChamplainPathLayerPrivate))
 
 enum
 {
@@ -66,12 +68,12 @@ enum
 static ClutterColor DEFAULT_FILL_COLOR = { 0xcc, 0x00, 0x00, 0xaa };
 static ClutterColor DEFAULT_STROKE_COLOR = { 0xa4, 0x00, 0x00, 0xff };
 
-//static guint signals[LAST_SIGNAL] = { 0, };
+/* static guint signals[LAST_SIGNAL] = { 0, }; */
 
 struct _ChamplainPathLayerPrivate
 {
   ChamplainView *view;
-  
+
   gboolean closed_path;
   ClutterColor *stroke_color;
   gboolean fill;
@@ -79,7 +81,7 @@ struct _ChamplainPathLayerPrivate
   gboolean stroke;
   gdouble stroke_width;
   gboolean visible;
-  
+
   ClutterGroup *content_group;
   ClutterActor *path_actor;
   GList *nodes;
@@ -104,7 +106,7 @@ champlain_path_layer_get_property (GObject *object,
 {
   ChamplainPathLayer *self = CHAMPLAIN_PATH_LAYER (object);
   ChamplainPathLayerPrivate *priv = self->priv;
-  
+
   switch (property_id)
     {
     case PROP_CLOSED_PATH:
@@ -134,7 +136,7 @@ champlain_path_layer_get_property (GObject *object,
     case PROP_VISIBLE:
       g_value_set_boolean (value, priv->visible);
       break;
-      
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -147,13 +149,11 @@ champlain_path_layer_set_property (GObject *object,
     G_GNUC_UNUSED const GValue *value,
     GParamSpec *pspec)
 {
-  ChamplainPathLayer *self = CHAMPLAIN_PATH_LAYER (object);
-  ChamplainPathLayerPrivate *priv = self->priv;
-  
   switch (property_id)
     {
     case PROP_CLOSED_PATH:
-      priv->closed_path = g_value_get_boolean (value);
+      champlain_path_layer_set_closed (CHAMPLAIN_PATH_LAYER (object),
+          g_value_get_boolean (value));
       break;
 
     case PROP_FILL:
@@ -182,10 +182,10 @@ champlain_path_layer_set_property (GObject *object,
       break;
 
     case PROP_VISIBLE:
-        champlain_path_layer_set_visible (CHAMPLAIN_PATH_LAYER (object),
-            g_value_get_boolean (value));
+      champlain_path_layer_set_visible (CHAMPLAIN_PATH_LAYER (object),
+          g_value_get_boolean (value));
       break;
-      
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -196,13 +196,13 @@ static void
 paint (ClutterActor *self)
 {
   ChamplainPathLayerPrivate *priv = GET_PRIVATE (self);
-  
+
   clutter_actor_paint (CLUTTER_ACTOR (priv->content_group));
 }
 
 
 static void
-pick (ClutterActor *self, 
+pick (ClutterActor *self,
     const ClutterColor *color)
 {
   ChamplainPathLayerPrivate *priv = GET_PRIVATE (self);
@@ -285,17 +285,17 @@ unmap (ClutterActor *self)
 }
 
 
-
 static void
 champlain_path_layer_dispose (GObject *object)
 {
   ChamplainPathLayer *self = CHAMPLAIN_PATH_LAYER (object);
   ChamplainPathLayerPrivate *priv = self->priv;
-  
+
+  if (priv->nodes)
+    champlain_path_layer_remove_all (CHAMPLAIN_PATH_LAYER (object));
+
   if (priv->view != NULL)
-    {
-      set_view (CHAMPLAIN_LAYER (self), NULL);
-    }
+    set_view (CHAMPLAIN_LAYER (self), NULL);
 
   if (priv->content_group)
     {
@@ -312,10 +312,9 @@ champlain_path_layer_finalize (GObject *object)
 {
   ChamplainPathLayer *self = CHAMPLAIN_PATH_LAYER (object);
   ChamplainPathLayerPrivate *priv = self->priv;
-  
+
   clutter_color_free (priv->stroke_color);
   clutter_color_free (priv->fill_color);
-  g_list_free (priv->nodes);
 
   G_OBJECT_CLASS (champlain_path_layer_parent_class)->finalize (object);
 }
@@ -329,12 +328,12 @@ champlain_path_layer_class_init (ChamplainPathLayerClass *klass)
   ChamplainLayerClass *layer_class = CHAMPLAIN_LAYER_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (ChamplainPathLayerPrivate));
-  
+
   object_class->finalize = champlain_path_layer_finalize;
   object_class->dispose = champlain_path_layer_dispose;
   object_class->get_property = champlain_path_layer_get_property;
   object_class->set_property = champlain_path_layer_set_property;
-  
+
   actor_class->get_preferred_width = get_preferred_width;
   actor_class->get_preferred_height = get_preferred_height;
   actor_class->allocate = allocate;
@@ -342,7 +341,7 @@ champlain_path_layer_class_init (ChamplainPathLayerClass *klass)
   actor_class->pick = pick;
   actor_class->map = map;
   actor_class->unmap = unmap;
-  
+
   layer_class->set_view = set_view;
   layer_class->get_bounding_box = get_bounding_box;
 
@@ -358,7 +357,8 @@ champlain_path_layer_class_init (ChamplainPathLayerClass *klass)
       g_param_spec_boolean ("closed",
           "Closed Path",
           "The Path is Closed",
-          FALSE, CHAMPLAIN_PARAM_READWRITE));
+          FALSE, 
+          CHAMPLAIN_PARAM_READWRITE));
 
   /**
    * ChamplainPathLayer:fill:
@@ -372,7 +372,8 @@ champlain_path_layer_class_init (ChamplainPathLayerClass *klass)
       g_param_spec_boolean ("fill",
           "Fill",
           "The shape is filled",
-          FALSE, CHAMPLAIN_PARAM_READWRITE));
+          FALSE, 
+          CHAMPLAIN_PARAM_READWRITE));
 
   /**
    * ChamplainPathLayer:stroke:
@@ -386,7 +387,8 @@ champlain_path_layer_class_init (ChamplainPathLayerClass *klass)
       g_param_spec_boolean ("stroke",
           "Stroke",
           "The shape is stroked",
-          TRUE, CHAMPLAIN_PARAM_READWRITE));
+          TRUE, 
+          CHAMPLAIN_PARAM_READWRITE));
 
   /**
    * ChamplainPathLayer:stroke-color:
@@ -430,7 +432,8 @@ champlain_path_layer_class_init (ChamplainPathLayerClass *klass)
       g_param_spec_double ("stroke-width",
           "Stroke Width",
           "The path's stroke width",
-          0, 100.0,
+          0, 
+          100.0,
           2.0,
           CHAMPLAIN_PARAM_READWRITE));
 
@@ -448,7 +451,6 @@ champlain_path_layer_class_init (ChamplainPathLayerClass *klass)
           "The path's visibility",
           TRUE,
           CHAMPLAIN_PARAM_READWRITE));
-
 }
 
 
@@ -456,7 +458,7 @@ static void
 champlain_path_layer_init (ChamplainPathLayer *self)
 {
   ChamplainPathLayerPrivate *priv;
-  
+
   self->priv = GET_PRIVATE (self);
   priv = self->priv;
   priv->view = NULL;
@@ -474,8 +476,7 @@ champlain_path_layer_init (ChamplainPathLayer *self)
   priv->content_group = CLUTTER_GROUP (clutter_group_new ());
   clutter_actor_set_parent (CLUTTER_ACTOR (priv->content_group), CLUTTER_ACTOR (self));
 
-  //TODO destroy + ref()
-  priv->path_actor =  clutter_cairo_texture_new (256, 256);
+  priv->path_actor = clutter_cairo_texture_new (256, 256);
   clutter_container_add_actor (CLUTTER_CONTAINER (priv->content_group), priv->path_actor);
 
   clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
@@ -513,10 +514,7 @@ add_node (ChamplainPathLayer *layer,
     gboolean append,
     guint position)
 {
-  ChamplainPathLayerPrivate *priv = GET_PRIVATE (layer);
-
-  g_return_if_fail (CHAMPLAIN_IS_PATH_LAYER (layer));
-  g_return_if_fail (CHAMPLAIN_IS_LOCATION (location));
+  ChamplainPathLayerPrivate *priv = layer->priv;
 
   g_signal_connect (G_OBJECT (location), "notify::latitude",
       G_CALLBACK (position_notify), layer);
@@ -525,7 +523,7 @@ add_node (ChamplainPathLayer *layer,
 
   if (append)
     priv->nodes = g_list_append (priv->nodes, location);
-  else  
+  else
     priv->nodes = g_list_insert (priv->nodes, location, position);
   schedule_redraw (layer);
 }
@@ -559,21 +557,21 @@ champlain_path_layer_add_node (ChamplainPathLayer *layer,
  *
  * Since: 0.10
  */
-void 
+void
 champlain_path_layer_remove_all (ChamplainPathLayer *layer)
 {
-  ChamplainPathLayerPrivate *priv = GET_PRIVATE (layer);
+  ChamplainPathLayerPrivate *priv = layer->priv;
   GList *elem;
-  
+
   g_return_if_fail (CHAMPLAIN_IS_PATH_LAYER (layer));
 
   for (elem = priv->nodes; elem != NULL; elem = elem->next)
     {
       GObject *node = G_OBJECT (elem->data);
-      
+
       g_signal_handlers_disconnect_by_func (node,
           G_CALLBACK (position_notify), layer);
-          
+
       g_object_unref (node);
     }
 
@@ -589,7 +587,7 @@ champlain_path_layer_remove_all (ChamplainPathLayer *layer)
  *
  * Gets a copy of the list of all #ChamplainLocation objects inserted into the layer. You should
  * free the list but not its contents.
- * 
+ *
  * Returns: (transfer container) (element-type ChamplainLocation): the list
  *
  * Since: 0.10
@@ -597,7 +595,7 @@ champlain_path_layer_remove_all (ChamplainPathLayer *layer)
 GList *
 champlain_path_layer_get_nodes (ChamplainPathLayer *layer)
 {
-  ChamplainPathLayerPrivate *priv = GET_PRIVATE (layer);
+  ChamplainPathLayerPrivate *priv = layer->priv;
 
   return g_list_copy (priv->nodes);
 }
@@ -616,8 +614,8 @@ void
 champlain_path_layer_remove_node (ChamplainPathLayer *layer,
     ChamplainLocation *location)
 {
-  ChamplainPathLayerPrivate *priv = GET_PRIVATE (layer);
-  
+  ChamplainPathLayerPrivate *priv = layer->priv;
+
   g_return_if_fail (CHAMPLAIN_IS_PATH_LAYER (layer));
   g_return_if_fail (CHAMPLAIN_IS_LOCATION (location));
 
@@ -640,7 +638,7 @@ champlain_path_layer_remove_node (ChamplainPathLayer *layer,
  *
  * Since: 0.10
  */
-void 
+void
 champlain_path_layer_insert_node (ChamplainPathLayer *layer,
     ChamplainLocation *location,
     guint position)
@@ -652,14 +650,12 @@ champlain_path_layer_insert_node (ChamplainPathLayer *layer,
 }
 
 
-
-
 static void
 relocate_cb (G_GNUC_UNUSED GObject *gobject,
     ChamplainPathLayer *layer)
 {
   g_return_if_fail (CHAMPLAIN_IS_PATH_LAYER (layer));
-    
+
   schedule_redraw (layer);
 }
 
@@ -674,7 +670,9 @@ redraw_path (ChamplainPathLayer *layer)
   ChamplainView *view = priv->view;
   gint x, y;
   guint last_width, last_height;
-  
+
+  priv->redraw_scheduled = FALSE;
+
   /* layer not yet added to the view */
   if (view == NULL)
     return FALSE;
@@ -683,25 +681,22 @@ redraw_path (ChamplainPathLayer *layer)
 
   if (!priv->visible || width == 0.0 || height == 0.0)
     return FALSE;
-    
+
   clutter_cairo_texture_get_surface_size (CLUTTER_CAIRO_TEXTURE (priv->path_actor), &last_width, &last_height);
 
-  if ((guint)width != last_width || (guint)height != last_height)
-    {
-      clutter_cairo_texture_set_surface_size (CLUTTER_CAIRO_TEXTURE (priv->path_actor), width, height);
-      clutter_actor_queue_relayout (CLUTTER_ACTOR (layer));
-    }
+  if ((guint) width != last_width || (guint) height != last_height)
+    clutter_cairo_texture_set_surface_size (CLUTTER_CAIRO_TEXTURE (priv->path_actor), width, height);
 
   champlain_view_get_viewport_origin (priv->view, &x, &y);
   clutter_actor_set_position (priv->path_actor, x, y);
 
   cr = clutter_cairo_texture_create (CLUTTER_CAIRO_TEXTURE (priv->path_actor));
 
-  /* Clear the drawing area */  
+  /* Clear the drawing area */
   cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
-  cairo_paint(cr);
+  cairo_paint (cr);
   cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-  
+
   for (elem = priv->nodes; elem != NULL; elem = elem->next)
     {
       ChamplainLocation *location = CHAMPLAIN_LOCATION (elem->data);
@@ -738,9 +733,6 @@ redraw_path (ChamplainPathLayer *layer)
 
   cairo_destroy (cr);
 
-  priv->redraw_scheduled = FALSE;
-  clutter_actor_queue_redraw (CLUTTER_ACTOR (layer));
-  
   return FALSE;
 }
 
@@ -768,33 +760,37 @@ redraw_path_cb (G_GNUC_UNUSED GObject *gobject,
 }
 
 
-static void 
+static void
 set_view (ChamplainLayer *layer,
     ChamplainView *view)
 {
   g_return_if_fail (CHAMPLAIN_IS_PATH_LAYER (layer) && (CHAMPLAIN_IS_VIEW (view) || view == NULL));
-  
+
   ChamplainPathLayer *path_layer = CHAMPLAIN_PATH_LAYER (layer);
-  
+
   if (path_layer->priv->view != NULL)
     {
       g_signal_handlers_disconnect_by_func (path_layer->priv->view,
-        G_CALLBACK (relocate_cb), path_layer);
+          G_CALLBACK (relocate_cb), path_layer);
+
+      g_signal_handlers_disconnect_by_func (path_layer->priv->view,
+          G_CALLBACK (redraw_path_cb), path_layer);
+
       g_object_unref (path_layer->priv->view);
     }
-  
+
   path_layer->priv->view = view;
 
   if (view != NULL)
     {
       g_object_ref (view);
-  
+
       g_signal_connect (view, "layer-relocated",
-        G_CALLBACK (relocate_cb), layer);
+          G_CALLBACK (relocate_cb), layer);
 
       g_signal_connect (view, "notify::latitude",
-        G_CALLBACK (redraw_path_cb), layer);
-        
+          G_CALLBACK (redraw_path_cb), layer);
+
       schedule_redraw (path_layer);
     }
 }
@@ -806,18 +802,18 @@ get_bounding_box (ChamplainLayer *layer)
   ChamplainPathLayerPrivate *priv = GET_PRIVATE (layer);
   GList *elem;
   ChamplainBoundingBox *bbox;
-  
+
   bbox = champlain_bounding_box_new ();
 
   for (elem = priv->nodes; elem != NULL; elem = elem->next)
     {
       ChamplainLocation *location = CHAMPLAIN_LOCATION (elem->data);
       gdouble lat, lon;
-      
-      g_object_get (G_OBJECT (location), "latitude", &lat, "longitude", &lon,
-          NULL);
-          
-      champlain_bounding_box_extend (bbox, lat, lon); 
+
+      lat = champlain_location_get_latitude (location);
+      lon = champlain_location_get_longitude (location);
+
+      champlain_bounding_box_extend (bbox, lat, lon);
     }
 
   if (bbox->left == bbox->right)
@@ -865,6 +861,25 @@ champlain_path_layer_set_fill_color (ChamplainPathLayer *layer,
 }
 
 
+/**
+ * champlain_path_layer_get_fill_color:
+ * @layer: a #ChamplainPathLayer
+ *
+ * Gets the path's fill color.
+ *
+ * Returns: the path's fill color.
+ *
+ * Since: 0.10
+ */
+ClutterColor *
+champlain_path_layer_get_fill_color (ChamplainPathLayer *layer)
+{
+  g_return_val_if_fail (CHAMPLAIN_IS_PATH_LAYER (layer), NULL);
+
+  return layer->priv->fill_color;
+}
+
+
 /**
  * champlain_path_layer_set_stroke_color:
  * @layer: a #ChamplainPathLayer
@@ -894,25 +909,6 @@ champlain_path_layer_set_stroke_color (ChamplainPathLayer *layer,
 }
 
 
-/**
- * champlain_path_layer_get_fill_color:
- * @layer: a #ChamplainPathLayer
- *
- * Gets the path's fill color.
- *
- * Returns: the path's fill color.
- *
- * Since: 0.10
- */
-ClutterColor *
-champlain_path_layer_get_fill_color (ChamplainPathLayer *layer)
-{
-  g_return_val_if_fail (CHAMPLAIN_IS_PATH_LAYER (layer), NULL);
-
-  return layer->priv->fill_color;
-}
-
-
 /**
  * champlain_path_layer_get_stroke_color:
  * @layer: a #ChamplainPathLayer
@@ -1083,7 +1079,7 @@ champlain_path_layer_set_visible (ChamplainPathLayer *layer,
  *
  * Since: 0.10
  */
-gboolean 
+gboolean
 champlain_path_layer_get_visible (ChamplainPathLayer *layer)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_PATH_LAYER (layer), FALSE);
@@ -1091,3 +1087,41 @@ champlain_path_layer_get_visible (ChamplainPathLayer *layer)
   return layer->priv->visible;
 }
 
+
+/**
+ * champlain_path_layer_set_closed:
+ * @layer: a #ChamplainPathLayer
+ * @value: TRUE to make the path closed
+ *
+ * Makes the path closed.
+ *
+ * Since: 0.10
+ */
+void
+champlain_path_layer_set_closed (ChamplainPathLayer *layer,
+    gboolean value)
+{
+  g_return_if_fail (CHAMPLAIN_IS_PATH_LAYER (layer));
+
+  layer->priv->closed_path = value;
+  g_object_notify (G_OBJECT (layer), "closed");
+}
+
+
+/**
+ * champlain_path_layer_get_closed:
+ * @layer: a #ChamplainPathLayer
+ *
+ * Gets information whether the path is closed.
+ *
+ * Returns: TRUE when the path is closed, FALSE otherwise
+ *
+ * Since: 0.10
+ */
+gboolean
+champlain_path_layer_get_closed (ChamplainPathLayer *layer)
+{
+  g_return_val_if_fail (CHAMPLAIN_IS_PATH_LAYER (layer), FALSE);
+
+  return layer->priv->closed_path;
+}
